      program ADJUST                                                             
c                                                                               
c  This program performs a moist convective adjustment. 
c  The determination of the local moist adiabat is based
c  moist static energy conservation .                                                   
c                                                                               
      parameter (n = 82)                                                           
      real t(n),q(n),p(n),z(n),wt(n),wp(n),wq(n)                          
      real t1(100),z1(100),q1(100),p1(n),wz(n)
c                                                                               
c  Definition of constants                                                         
c 
      data rgas,g/287.05, 9.81/                                                                             
      rg        = rgas/g                                                             
c                                                                               
c  Read input data from top to bottom.                                           
c    file temp.dat contains : pressure and temperature                              
c    file spec.dat contains : pressure and specific humdity                         
c    file hght.dat contains : pressure and geopotential                             
c                                                                               
      open (1,file='temp.dat',status='old')                                     
      open (2,file='spec.dat',status='old')                                     
      open (3,file='hght.dat',status='old') 
c                                    
      do 6300 k = 1, n                                                               
         read(1,101) p(k) ,t(k)                                                      
         read(2,102) p(k) ,q(k)                                                      
         read(3,102) p(k) ,z(k)                                                      
6300  continue                                                                  
 101  format(2f10.2)                                                            
 102  format(2f10.5)                                                            
c                                                                               
c  Invert data to read from  bottom to top .                               
c                                                                               
      do 6302 k = 1, n                                                               
         kk     = n-k+1                                                            
         wp(k)  = p(kk)                                                          
         wt(k)  = t(kk)                                                          
         wq(k)  = q(kk)                                                          
         wz(k)  = z(kk)                                                          
 6302 continue
      ps        = wp(1)
c                                                                               
c  Determine the top of the adjustement                                        
c                                                                               
      call ADJTOP (wz,wt,wq,wp,p0,z0,t0,q0,n)                                   
c                                                                                
         if (p0.ne.0.) then
c
      write (6,1000)p0,t0,z0,q0
 1000 format(/,2x,'reference level                     ',f7.2,/,
     &       2x,'temperature at reference level      ',f7.2,/,
     &       2x,'height at reference level           ',f7.2,/,                         
     &       2x,'specific humidityat reference level ',f7.4  )
c                                  
      nl        = int ((ps-p0)/10) + 1 
      write (6,1001)nl                                                   
 1001 format(2x,'number of levels to be adjusted     ',i7,/)                           
c                                                                               
c  Perform the adjustment of the layer below the                                  
c  reference level.                                                            
c                                                                               
      call MOIST (z0,t0,q0,p0,0.05,-10.,nl,z1,t1,q1,p1)
c                         
         endif                                                                     
c                                                                               
c  Display the original and the adjusted soundings 
c  below the reference level.                                
c
      write (6,1002)
      write (6,1003)
      write (6,1004)
 1002 format(30x,'below adjustment top' ,/)                         
 1003 format(15x,'original sounding',22x,'adjusted sounding',/ )                         
 1004 format(4x,2('pressure',2x,'height',3x,'temp.',4x,'spec. hum.'
     &,2x),/)                         
 1005 format(4x,3(f7.2,2x),f7.4,5x,3(f7.2,2x),f7.4)                         
      do 6304 k = n-nl+1,n
         kk     = k-(n-nl)                                                          
      write (6,1005)p(k),z(k),t(k),q(k),p1(kk),z1(kk),t1(kk),q1(kk)
 6304 continue                                               
      stop                                                                      
      end                                                                       
      subroutine MOIST (z0,t0,q0,p0,dt,dp,nl,z,t,q,p1)                           
c                                                                               
c  This subroutine constructs a moist adiabat from 
c  known thermodynamic temperature (t), specific                           
c  humidity (q), and geopotential height (z) at a                                  
c  reference level p.                                                           
c  It involves the computations of the saturated specific                       
c  humidity (qs) at the temperature (ts). and evaluate the                          
c  geopotential height (z) at the pressure level of ts and
c  qs.  using the hydrostatic relation.
c  The computation uses the invariance of the moist static 
c  energy in a  moist adiabatic process .                       
c  The routine assumes that the air is saturated .                        
c                                                                               
c                                                                               
c  input parameters   :                                                        
c                                                                               
c   z0            : height in meters of p0 surface                                        
c   t0            : temperature in degrees celcius at p0 surface                          
c   q0            : specific humidity in gram per gram at p0                              
c   p0            : pressure in millibars of reference surface                            
c   dt            : temperature increment in degrees celcius for                          
c                   successive guesses of temperature at p surface                        
c   dp            : pressure change between desired pressure surfaces                     
c                                                                               
c  Note   : the subroutine is written to start at p0 and work                      
c           upward through decreasing pressure. It can also work
c           downward through increasing pressure by setting dp 
c           negative. If this is done the order of the check                     
c           if(et-em)  should be reversed by making to read if(em-et).                    
c                                                                               
c                                                                               
c  output parameters   : returns z, t and q at nl levels                                       
c
c   nl            : number of pressure surfaces in question (includes p0)                 
c   z             : height in meters at each pressure level                                
c   t             : temperature in degrees celcius at each pressure level                  
c   q             : specific humidity at each pressure level                               
c                                                                               
c                                                                               
      real z(nl),t(nl),q(nl),p1(nl)                                        
c                                                                               
c  Set constants:                                                               
c
      data g,cp,hl,a,b/9.81,1004.,2.49e06,17.27,35.86/                                                                              
      r         = 2000./7.                                                               
      ia        = 1000
c                                                                  
c  Set first element in array equal to given values
c                             
       z(1)     = z0                                                                  
       t(1)     = t0                                                                  
       q(1)     = q0
c                                                                  
c  Compute the reference moist static energy . 
c                                         
      em        = g*z(1) + cp*t(1) + hl*q(1)                                                 
      n         = nl                                                                       
      p1(1)     = p0
c                                                                
c  Begin main processing loop
c                                                   
      do 6320 l = 2, n 
c                                                             
c  Increment pressure
c                                                           
          p     = p0-((l-1)*dp)
c                                                           
c  Begin temp, humidity, height loop for this level.                            
c  An iterative procedure determines the level                              
c  where the moist static energy is equal to that of
c  the reference level .
c                                                              
      do 6321 k = 1, ia
c                                                             
c  Guess temperature
c                                                            
         t(l)   = t(l-1)-(k*dt)
c                                                        
c  Compute saturation vapor pressure and specific 
c  humidity for this temperature .
c                                                        
         es     = 6.11*exp(a*(t(l)-273.16)/(t(l)-b))                                     
         q(l)   = (0.622*es)/(p-0.378*es)
c                                              
c  compute averaged quantities for use in                                       
c  height (thickness) calculation
c                                               
          pbar  = p                                                                  
          tbar  = (t(l)+t(l-1))*0.5                                                 
          qbar  = (q(l)+q(l-1))*0.5                                                 
          dz    = (r*tbar*(1.+0.61*qbar)*dp)/(g*pbar)                                    
          z(l)  = z(l-1)+dz
c                                                            
c  Compute static energy 
c                                                       
          et    = g*z(l)+cp*t(l)+hl*q(l)
c                                                 
c  if static energy not the same as that at p0, guess a new
c  temperature else go to the next level.
c  Note : If the subroutine is used with decreasing pressure
c  use the commented if statement.                      
cc    if(et-em)6320,6320,6321                                                      
      if(em-et)6321,6321,6320                                                      
 6321 continue                                                                  
 6320 p1(l)     = p                                                                 
      continue                                                                  
      return                                                                    
      end 
      subroutine ADJTOP (z,t,q,p,po,zo,to,qo,n)                                  
c                                                                               
c  This subroutine determines the top of moist convective                        
c  adjustment and returns the reference pressure level                           
c  with the corresponding reference height, temperature,                         
c  and specific humidity .                                                        
c  the check is done from top to bottom                                          
c  incrementing by 10 mb                                                          
c                                                                               
c  Input parameters    :                                                            
c   p             : pressure in millibars                                     
c   z             : height in geopotential meters                             
c   t             : temperature in degrees kelvin                             
c   q             : specific humidity in gm/gm                                
c   n             : number of pressure levels                                 
c                                                                               
c  Iutput parameters   :                                                           
c   po            : reference pressure level in millibars                    
c   to            : reference temperature in degrees kelvin                  
c   zo            : reference height in geopotential meters                  
c   qo            : reference specific humidity in gm/gm                     
c                                                                               
c  definition           :                                               
c   em            : moist static energy in joules/kgm                               
c   eadj          : static energy at convective top                                 
c   fct           : conversion factor                                               
c   rinc          : pressure increment in millibars                                 
c   fg            : pressure of first guess                                         
c   npt           : number of levels from surface to top                            
c                   of convective adjustment                                        
c                                                                               
      real z(n),t(n),q(n),p(n)                                             
      data g,cp,hl/9.81,1004.,2.49e06/                                          
      fct       = 1000./g                                                               
      nt        = 0                                                                      
      rinc      = 10.                                                                  
      fg        = 300.                                                                   
      pt        = ((p(1)-fg)/rinc)+1.                                                    
      npt       = int(pt)                                                               
   15 continue                                                                  
      em        = 0.                                                                     
      do 6330 k = 1, npt                                                             
         xx     = g*z(k)+cp*t(k)+hl*q(k)                                                 
         em     = em+xx                                                                  
 6330 continue                                                                  
      em        = em*fct                                                                 
      dp        = float(npt-1)*10.*100.                                                  
      eadj      = (g*z(npt)+cp*t(npt)+hl*q(npt))*dp/g                                  
      dif       = em-eadj
      print 1100,npt,em,eadj,dif
      if (em.lt.eadj) go to 20                                                    
      npt       = npt-1                                                                 
      nt        = nt+1                                                                   
      if (nt.gt.100) go to 99                                                     
      go to 15                                                                  
   20 if (nt.ne.0) go to 22                                                       
      po        = p(1)                                                                   
      zo        = z(1)                                                                   
      to        = t(1)                                                                   
      qo        = q(1)                                                                   
      go to 23                                                                  
   22 continue                                                                  
      npt       = npt+1                                                                 
      po        = p(npt)                                                                 
      zo        = z(npt)                                                                 
      to        = t(npt)                                                                 
      qo        = q(npt)                                                                 
   23 return                                                                    
 1100 format(1x,i3,2x,'em',e12.4,2x,'eadj',e12.4,2x,
     & 'em-eadj',e12.4)            
   99 print 1200                                                                
 1200 format(1x,'no convective adjustment done')                                
      return                                                                    
      end                                                                       
