      program L_C_L
c                                                                               
c  This program detemines the lifting condensation level                            
c  given surface parameters.the lcl is obtained by                               
c  raising a parcel of air from the surface to a level                           
c  where it reachs saturation.the technique used is                         
c  based on the conservation of moist static energy.
c
c  definition of surface parameters (pressure,temperature
c  and specific humidity)                                                    
c
      p0        = 1011.0                                                                 
      t0        = 296.0                                                                 
      q0        = 0.0170                                                                 
c                                                                               
c  Subroutine LCL determines the lcl.                                            
c                                                                               
      call LCL (p0,t0,q0,plcl,zlcl,tlcl,qlcl,n)                                  
      write (6,1000)n                                                             
      write (6,1001)                                                              
      write (6,1002)tlcl,plcl,zlcl,qlcl                                           
1000  format(10x,'number of itterations = ',i5,/)                               
1001  format(6x,'tlcl',4x,'plcl',5x,'zlcl',4x,'qlcl',/)                         
1002  format(3x,3f8.1,f8.4)                                                     
      stop                                                                      
      end                                                                       
      subroutine LCL (p0,t0,q0,plcl,zlcl,tlcl,qlcl,n)                            
c                                                                               
c  This subroutine computes the level of condensation                                 
c  starting from the top moving downwards with a small
c  increment in pressure change .It checks where saturation 
c  occurs when the parcel is lifted along the dry adiabatic                                  
c  lapse rate. At saturation q0 = qs at temperature ts                             
c  at the lcl level . The dry static energy is conserved .                                                                
c                                                                   
      dp        = 1.                                                                     
      n         = 0                                                                       
      plcl      = 300.                                                                 
    4 tlcl      = t0*((plcl/1000.)**0.286)                                             
      es        = 6.11*exp(25.22*(1.-273.16/tlcl))
     &                      *((273.16/tlcl)**5.33)                 
      qs        = 0.622*es/(plcl-(0.379*es))                                             
      eps       = q0 - qs                                                                 
      if (eps.le.0.) go to 3                                                     
      plcl      = plcl+dp                                                              
      n         = n+1                                                                     
      go to 4
    3 zlcl      = (1004./9.81)*(t0-tlcl)                                               
      qlcl      = qs                                                                   
      return                                                                    
      end                                                                       
